@@ -47,16 +47,23 @@ namespace GlomConversions
   tm parse_time(const Glib::ustring& text, bool& success);
   tm parse_time(const Glib::ustring& text, const std::locale& locale, bool& success);
 
-  Glib::ustring format_tm(const tm& tm_data, const std::locale& locale, char format);
+  Glib::ustring format_tm(const tm& tm_data, const std::locale& locale, const char* format);
   //static tm parse_tm(const Glib::ustring& text, const std::locale& locale, char format);
 
   bool value_is_empty(const Gnome::Gda::Value& value);
   Gnome::Gda::Value get_empty_value(Field::glom_field_type field_type);
 
+  /** This is like get_empty_value(), but it never returns a NULL-type object, 
+   * which would be useless to calculations.
+   */
+  Gnome::Gda::Value get_empty_value_suitable_for_python(Field::glom_field_type field_type);
+
   Gnome::Gda::Value get_example_value(Field::glom_field_type field_type);
-  
+
   Glib::ustring get_escaped_binary_data(guint8* buffer, size_t buffer_size);
   Gnome::Gda::Value parse_escaped_binary_data(const Glib::ustring& escaped_data);
+
+  Gnome::Gda::Value convert_value(const Gnome::Gda::Value& value, Field::glom_field_type target_glom_type);
 }
 
 //Copied from Postgres's PQunescapeBytea() so I don't have the trouble of finding and linking to the 
