@@ -32,32 +32,51 @@ public:
   virtual ~Notebook_Data();
 
   //Create the layout for the database structure, and fill it with data.
-  virtual bool init_db_details(const Glib::ustring& strTableName, const Glib::ustring& strWhereClause = Glib::ustring());
+  virtual bool init_db_details(const FoundSet& found_set);
+
+  ///Get the existing where clause, previously supplied to init_db_details().
+  FoundSet get_found_set() const;
+
+  ///Show the details for a particular record, without affecting the list view:
+  virtual void show_details(const Gnome::Gda::Value& primary_key_value);
 
   virtual void select_page_for_find_results(); //Details for 1, List for > 1.
 
   virtual void do_menu_developer_layout(); //override
   virtual void do_menu_file_print(); //override
 
+  void get_record_counts(gulong& total, gulong& found);
+
   enum dataview
   {
     DATA_VIEW_Details,
     DATA_VIEW_List
   };
 
-  virtual dataview get_current_view() const;
+  dataview get_current_view() const;
+  void set_current_view(dataview view);
+
+  typedef sigc::signal<void, const Glib::ustring&, Gnome::Gda::Value> type_signal_record_details_requested;
+  type_signal_record_details_requested signal_record_details_requested();
 
 protected:
 
+  ///Show the counts of all records and found records:
+  void update_records_count();
+
   //Signal handlers:
-  virtual void on_list_user_requested_details(Gnome::Gda::Value primary_key_value);
-  virtual void on_Details_user_requested_related_details(Glib::ustring strTableName, Gnome::Gda::Value primary_key_value);
+  virtual void on_list_user_requested_details(const Gnome::Gda::Value& primary_key_value);
+  void on_Details_user_requested_related_details(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value);
+
+  virtual void on_switch_page_handler(GtkNotebookPage* pPage, guint uiPageNumber);
 
   //Member widgets:
   Box_Data_List m_Box_List;
   Box_Data_Details m_Box_Details;
 
   guint m_iPage_Details, m_iPage_List;
+  Glib::ustring m_table_name;
+  type_signal_record_details_requested m_signal_record_details_requested;
 };
 
 #endif
